@@ -9,7 +9,7 @@ from transparentpath import TransparentPath
 import logging
 
 from .configs import NodePublicConfig, CentralConfig
-from .aggregations import adaboost_aggregation
+from .aggregations import AdaBoostAggregation
 
 logger = logging.getLogger(__name__)
 
@@ -48,15 +48,16 @@ class NodeGate:
 
     instances = 0
 
-    def __init__(self, node_config_path: TransparentPath):
+    def __init__(self, node_config_path: Union[str, TransparentPath]):
         """
         Parameters
         ----------
-        node_config_path: TransparentPath
+        node_config_path: Union[str, TransparentPath]
             Directory were node's public configuration file can be found
         """
-        if not isinstance(node_config_path, TransparentPath):
-            raise TypeError(f"node_config_path should be a transparentpath, got {type(node_config_path)} instead")
+
+        if type(node_config_path) == str:
+            node_config_path = TransparentPath(node_config_path)
 
         self.id = NodeGate.instances
         self.node_config_path = node_config_path
@@ -151,22 +152,20 @@ class CentralServer:
         List of all gates to the nodes the central server should monitor.
     ruleset: RuleSet
         Central server model's ruleset
-    aggregation: str
-        Name of the aggregation method. Can be one of: \n
-          * adaboost\n
+    aggregation: Aggregation
+        Instance of one of the `ifra.aggregations.Aggregation` daughter classes.
     """
 
     possible_aggregations = {
-        "adaboost": adaboost_aggregation
+        "adaboost_aggregation": AdaBoostAggregation
     }
     """Possible string values and corresponding aggregation methods for *aggregation* attribute of
     `ifra.central_server.CentralServer`"""
 
     def __init__(
         self,
-        nodes_configs_paths: List[TransparentPath],
+        nodes_configs_paths: List[Union[str, TransparentPath]],
         central_configs_path: Union[str, Path, TransparentPath],
-        aggregation: str
     ):
         """
         Parameters
@@ -175,9 +174,6 @@ class CentralServer:
             List of remote paths pointing to each node's public configuration file
         central_configs_path: Union[str, Path, TransparentPath]
             Central server configuration. See `ifra.configs.CentralConfig`
-        aggregation: str
-            Name of the aggregation method. Can be one of: \n
-              * adaboost (see `ifra.aggregations.adaboost_aggregation`)\n
         """
         self.reference_node_config = None
 
@@ -191,15 +187,15 @@ class CentralServer:
 
         self.ruleset = None
 
-        if aggregation not in self.possible_aggregations:
-            s = f"Aggregation '{aggregation}' is not known. Can be one of:"
-            "\n".join([s] + list(self.possible_aggregations.keys()))
+        if self.central_configs.aggregation not in self.possible_aggregations:
+            s = f"Aggregation '{self.central_configs.aggregation}' is not known. Can be one of:"
+            s = "\n".join([s] + list(self.possible_aggregations.keys()))
             raise ValueError(s)
-        self.aggregation = aggregation
+        self.aggregation = self.possible_aggregations[self.central_configs.aggregation](self)
 
     def aggregate(self, rulesets: List[RuleSet]) -> Tuple[str, Union[RuleSet, None]]:
         """Aggregates rulesets in `ifra.central_server` *ruleset* using
-        `ifra.central_server.aggregation`
+        `ifra.central_server.CentralServer` *aggregation*
         
         Parameters
         ----------
@@ -219,7 +215,7 @@ class CentralServer:
             rulesets, len(self.nodes), self.ruleset, self.central_configs.max_coverage
         )
 
-    def watch(self, timeout: Optional[int], sleeptime: int = 5):
+    def watch(self, timeout: int = 0, sleeptime: int = 5):
         """Monitors new changes in the nodes, every ''sleeptime'' seconds for ''timeout'' seconds, triggering
         aggregation and pushing central model to nodes when enough new node models are available.
 
@@ -228,16 +224,16 @@ class CentralServer:
         Parameters
         ----------
         timeout: Optional[int]
-            How long should the watch last. If None, will last until all nodes gave a model but no new rules could be
-            found from them.
+            How many seconds should the watch last. If <= 0, will last until all nodes gave a model but no new rules
+            could be found from them. Default value = 0.
         sleeptime: int
-            How long between each checks for new nodes models
+            How many seconds between each checks for new nodes models. Default value = 5.
         """
         t = time()
         updated_nodes = []
         rulesets = []
         learning_over = False
-        while time() - t < timeout:
+        while time() - t < timeout or timeout <= 0:
             new_models = False
 
             for node in self.nodes:
